# Filtre d'amlioration des index produits par Latex avec la macro \makeindex (fichier .idx)
# en vue d'un traitement (tri + compactage des numros de page) par la commande makeindex :
# mise en forme des \indexentry pour que le tri des mots accentus par makeindex marche
# pour le franais. 
# On peut mettre des commentaires introduits par `%' dans le fichier .idx qui sont supprims
# dans le fichier rsultat.

# Exemple :A
# avant :
#   \indexentry{algbrique}{292}
#   \indexentry{Algbrique@\textit{algbrique}}{292}
#   \indexentry{"@@ (commande "@)}{155}% essai
# aprs :
#   \indexentry{algebrique@algbrique}{292}
#   \indexentry{algebrique@\textit{algbrique}}{292}
#   \indexentry{"@@ (commande "@)}{155}% essai

BEGIN{
   FS=""
	init_table()
}

/^[ 	]*%/{next}


{
    # suppression du commentaire ventuel
    gsub(/%[^%]*$/,"")

    # sauvegarde du numro de page
		 if ( (i=match($0,/}{[0-9][0-9]*}$/)) == 0){
           error("le numro de ligne")
       }
     numero = substr($0,i,RLENGTH) # sauvegarde le numro de ligne soaus la forme ``}{295}''
    gsub(/}{[0-9][0-9]*}$/,"")      # supprime de $0 le numro sauvegard
	
    # suppression du dbut de ligne \indexentry{
    gsub(/^[ 	]*\\indexentry{/,"") # supprime de $0 le dbut de ligne

    # sauvegarde des @ et des ! ventuels quots par "
    gsub(/"@/,"\001")               # inhibe les @ quots par "
    gsub(/"!/,"\002")               # inhibe les ! quots par "
    
    # sparation des champs key@field
		 if ( (i=match($0,/@.*$/)) == 0){ # pas de sparateur @
          key0  = $0
          field = key0
			 } else {
          key0  = substr($0,1, index($0,"@")-1)
          field = substr($0,i+1,RLENGTH-1)
        }

    # traitement des sous-index spars par !
     gsub(/^[^!]*!/,"",field)          #supprime la partie rpte avant des sous-index
    
    # sparation des niveaux de la cl
		 if ( (i=match(key0,/!.*$/)) == 0){ # pas de sparateur !
          key0a  = key0
          key0b  = ""
          sous_index =1
			 } else {
          key0a  = substr(key0,1, index(key0,"!")-1)
          key0b  = substr($key0,i+1,RLENGTH-1)
        	 if ( length(key0b) == 0) sous_index = 2
          else sous_index = 3
        }
    # restitution des caractres @ et ! quots
     gsub(/\001/,"\"@",key0a)    #restitue les @ quots par "
     gsub(/\002/,"\"!",key0a)    #restitue les ! quots par "
     gsub(/\001/,"\"@",key0b)    #restitue les @ quots par "
     gsub(/\002/,"\"!",key0b)    #restitue les ! quots par "
     gsub(/\001/,"\"@",field)    #restitue les @ quots par "
     gsub(/\002/,"\"!",field)    #restitue les ! quots par "

 # traduction des lettres accentues de key0 dans key1     
    stripkey(key0a) ; key1a=newkey
    stripkey(key0b) ; key1b=newkey
	 if (sous_index == 1 ){
       printf("\\indexentry{%s<%s>@%s%s\n", key1a, key0a, field, numero)
    }else if (sous_index == 2) {
       printf("\\indexentry{%s<%s>!@%s%s\n", key1a, key0a, field, numero)
    }else { # == 3
       printf("\\indexentry{%s<%s>!%s<%s>@%s%s\n", key1a, key0a, key1b, key0b, field, numero)
    }
}

function error(mess){
    printf("@@ mkindex.awk : erreur de syntaxe dans %s, ligne %d\n", mess, NR) | "cat 1>&2" 
	 exit 1;
}

function stripkey(s) {
 # rend dans newkey le rsultat de la transformation de s : 
 #  -> e  -> e, suppression des signes de ponctuation (non significatifs pour le tri)
	 newkey=""
    for (i=1; i <= length(s); i++) {
       c= substr(s, i, 1)
       nc=t[c]
#       if (nc == "")
#          nc=c
#       else if (nc == "\003") # void
#          nc=""
       newkey= newkey nc
	  }
}

function init_table() {
t[""]="a" ; t[""]="a" ;
t[""]="a" ; t[""]="a" ;
t[""]="e" ; t[""]="e" ;
t[""]="e" ; t[""]="e" ;
t[""]="e" ; t[""]="e" ;
t[""]="e" ; t[""]="e" ;
t[""]="e" ; t[""]="e" ;
t[""]="i" ; t[""]="i" ;
t[""]="i" ; t[""]="i" ;
t[""]="o" ; t[""]="o" ;
t[""]="u" ; t[""]="u" ;
t[""]="u" ; t[""]="u" ;
t[""]="u" ; t[""]="u" ;
t[""]="c" ; t[""]="c" ;

t["a"]="a" ; t["A"]="a" ;
t["b"]="b" ; t["B"]="b" ;
t["c"]="c" ; t["C"]="c" ;
t["d"]="d" ; t["D"]="d" ;
t["e"]="e" ; t["E"]="e" ;
t["f"]="f" ; t["F"]="f" ;
t["g"]="g" ; t["G"]="g" ;
t["h"]="h" ; t["H"]="h" ;
t["i"]="i" ; t["I"]="i" ;
t["j"]="j" ; t["J"]="j" ;
t["k"]="k" ; t["K"]="k" ;
t["l"]="l" ; t["L"]="l" ;
t["m"]="m" ; t["M"]="m" ;
t["n"]="n" ; t["N"]="n" ;
t["o"]="o" ; t["O"]="o" ;
t["p"]="p" ; t["P"]="p" ;
t["q"]="q" ; t["Q"]="q" ;
t["r"]="r" ; t["R"]="r" ;
t["s"]="s" ; t["S"]="s" ;
t["t"]="t" ; t["T"]="t" ;
t["u"]="u" ; t["U"]="u" ;
t["v"]="v" ; t["V"]="v" ;
t["w"]="w" ; t["W"]="w" ;
t["x"]="x" ; t["X"]="x" ;
t["y"]="y" ; t["Y"]="y" ;
t["z"]="z" ; t["Z"]="z" ;
t["0"]="0" ;

t["0"]="0" ;
t["1"]="1" ;
t["2"]="2" ;
t["3"]="3" ;
t["4"]="4" ;
t["5"]="5" ;
t["6"]="6" ;
t["7"]="7" ;
t["8"]="8" ;
t["9"]="9" ;

#t[" "]="\003" ; t["("]="\003" ; t[")"]="\003" ;
#t[","]="\003" ; t["["]="\003" ; t["]"]="\003" ;
#t["'"]="\003" ; t[""]="\003" ; t[""]="\003" ;
#t["-"]="\003" ; t["."]="\003" ; t[";"]="\003" ;
}
